// BattleTruck Showdown
// alpha v0.02
// contest entry by Eben Olson baniel@crosswinds.net
// begun 4/23/2000
// revision history:
// v0.01 Loaded all BattleTruck .bmps
// v0.02 revised sprite system, added sprs for BattleTruck
// v0.03 removed old spcfght stuff !TRIUMPHED OVER CRASH BUG!
// v0.04 Fixed camera system
// v0.05 Added particle system
// v0.06 Implemented cannon, rocketrack and nuke
// 4/24/2000: v0.06
// v0.07 tweaked weapons, added logic 10ms cycle assurance timer
// v0.08 added background, walls, laser weapon
// v0.09 added bazooka and machine gun, napalm, car<->map collisions, exhaust
// 4/25/2000: v0.09
// v0.10 added zoom system, fixed wall collisions, background changes
// 4/26/2000: v0.10
// v0.11 added mines and oil, changed wall collisions
// 4/27/2000: v0.11
// v0.12 fixed wall collisions again, added weapon selection keys
// 4/28/2000: v0.12
// v0.13 added side panels
// v0.14 added armor, damage, death, respawning
// v0.15 added shield
// v0.16 fixed shield, upgraded panels
// v0.17 reordered main loop, added money, tuneup and pitstop, added shopping
// 4/29/2000: v0.17
// v0.18 made new bg, made bg map editor, added map loading
// v0.19 began encapsulation
// v0.20 completed encapsulation, finished implementing specials, fixed ammo rollover
// v0.21 fixed special displays, added bg music
// v0.22 began menu
// v0.23 finished menu !!! FINAL !!!

#include <allegro.h>
#include <math.h>

#define PLAYFIELDW 1152
#define PLAYFIELDH 1152

#define MACHINE_GUN 0
#define ROCKET_RACK 1
#define CANNON      2
#define NAPALM      3
#define LASER       4
#define BAZOOKA     5
#define NUKE        6

#define NONE        0
#define MINE        1
#define OIL         2
#define SHIELD      3
#define BOT         4

#define BLOCKSIZE   32

#define LEFT    2
#define RIGHT   4
#define TOP     8
#define BOTTOM  16
#define CENTER  32

#define pi 3.1415926

#define scale_x(ang) (float)(cos((ang - 90) * pi / 180))
#define scale_y(ang) (float)(sin((ang - 90) * pi / 180))

char map[PLAYFIELDW/BLOCKSIZE][PLAYFIELDH/BLOCKSIZE];

PACKFILE *mapfile;

BITMAP *scrbuf, *coldetmap, *coldetmap2;

BITMAP *panel[4];
BITMAP *healthbar;

BITMAP *background;

BITMAP *car1, *car2, *car3, *car4, *carexp;
BITMAP *bazooka, *laser, *cannon, *nuke, *rocketrack, *napalm, *machinegun;
BITMAP *bazexp, *lasexp, *cannexp, *nukeexp, *rrexp, *napexp, *mineexp;
BITMAP *mine, *oil, *shield, *bot;
BITMAP *fixup, *pitstop, *smallmoney, *bigmoney, *ammo;

BITMAP *backg, *lowrezbg;

PALETTE pal;

int fps, count, gologic = 0, done = 0;
char textcol = 48;

int nomag = 0, nomusic = 0, moneyrate = 1000, ammorate = 2000, fixuprate = 1000;
int mag = 1;

void fps_timer()
{
  fps = count; count = 0;
}
END_OF_FUNCTION(fps_timer);

void logic_timer()
{
  gologic = 1;
}

END_OF_FUNCTION(logic_timer);

int showboxes = 0;

class ent;
class OBJ;
class player;
class sprite;
class particle;

typedef void ACTION (OBJ *objp);
typedef ACTION *ACTIONp;

typedef void particleACTION (particle *objp);
typedef particleACTION *particleACTIONp;

typedef void PlayerACTION (player *plr);
typedef PlayerACTION *PlayerACTIONp;

void machinegungo(OBJ *obj);
void rocketrackgo(OBJ *obj);
void rrexpgo(OBJ *obj);
void cannongo(OBJ *obj);
void cannexpgo(OBJ *obj);
void bazookago(OBJ *obj);
void bazexpgo(OBJ *obj);
void nukego(OBJ *obj);
void nukeexpgo(OBJ *obj);
void laser_spawnergo(OBJ *obj);
void lasergo(OBJ *obj);
void lasexpgo(OBJ *obj);
void napalmgo(OBJ *obj);
void napexpgo(OBJ *obj);
void minego(OBJ *obj);
void mineexpgo(OBJ *obj);
void oilgo(OBJ *obj);
void carexpgo(OBJ *obj);
void nukesmokego(OBJ *obj);
void shieldgo(OBJ *obj);
void shieldballgo(OBJ *obj);
void pitstopgo(OBJ *obj);
void fixupgo(OBJ *obj);
void smallmoneygo(OBJ *obj);
void bigmoneygo(OBJ *obj);
void ammogo(OBJ *obj);

void par_flamego(particle *objp);
void par_ashgo(particle *objp);
void par_lasergo(particle *objp);
void par_bulletgo(particle *objp);
void par_shieldsparkgo(particle *objp);
void par_botsparkgo(particle *objp);

void init_system();
void check_collisions();
void update_panels(player plr1, player plr2);
int colcheck(ent *obj1, ent *obj2);
int mapcolcheck(ent *obj);
int load_level(char *fname);
void palrot();
void playgame(int car1, int car2, int map);
int do_menu();

/****************************************************************************/
////////////////////////////////// CLASS sprite///////////////////////////////
/****************************************************************************/

class sprite
{
public:
  BITMAP **grf;
  int *radius;

  void create(BITMAP *bmp, int frames);
  void draw(BITMAP *bmp, int x, int y, int cx, int cy, int angle, int frame);
  void drawsmall(BITMAP *bmp, int x, int y, int cx, int cy, int angle, int frame);
  int getradius(BITMAP *bmp);
};

void sprite::create(BITMAP *bmp, int frames)
{
  int i;

  grf = new BITMAP*[frames];
  radius = new int[frames];

  for (i=0; i<frames; i++)
  {
    grf[i] = create_sub_bitmap(bmp, i*(bmp->w/frames), 0, (bmp->w/frames), bmp->h);
    radius[i] = getradius(grf[i]);
  }
}

void sprite::draw(BITMAP *bmp, int x, int y, int cx, int cy, int angle, int frame)
{
  int rx, ry;
  rx = (int)(x-cx);
  ry = (int)(y-cy);
  rx -= grf[frame]->w/2;
  ry -= grf[frame]->h/2;
  rotate_sprite(bmp, grf[frame], rx, ry, itofix(angle*0.711));
  if (showboxes) circle(bmp, rx+grf[frame]->w/2, ry+grf[frame]->h/2, radius[frame], 81);
}


void sprite::drawsmall(BITMAP *bmp, int x, int y, int cx, int cy, int angle, int frame)
{
  int rx, ry;
  rx = (int)(x-cx);
  ry = (int)(y-cy);
  rx -= grf[frame]->w/2;
  ry -= grf[frame]->h/2;
  rx *= 0.5; ry *= 0.5;
  rotate_scaled_sprite(bmp, grf[frame], rx, ry, itofix(angle*0.711), ftofix(0.5));
  if (showboxes) circle(bmp, rx+grf[frame]->w/2, ry+grf[frame]->h/2, radius[frame], 81);
}

int sprite::getradius(BITMAP* bmp)
{
  int rad, x, y, ang;
  BITMAP *bmp2;

  textout_centre(screen, font, "Calculating Collision Radii", 320, 100, textcol);
  textcol++;
  if (textcol > 63) textcol = 48;

  if (bmp->w > bmp->h) rad = bmp->w;
  else rad = bmp->h;

  bmp2 = create_bitmap(rad, rad);
  clear(bmp2);
  blit(bmp, bmp2, 0, 0, abs(bmp->w-rad)/2, abs(bmp->h-rad)/2, bmp->w, bmp->h);

  while (rad > 0)
  {
    for (ang=0; ang<360; ang++)
    {
      putpixel(screen, cos((ang-90)*pi/180)*25+320, sin((ang-90)*pi/180)*25+200, 0);
      x = cos(ang*pi/180)*rad + bmp2->w/2;
      y = sin(ang*pi/180)*rad + bmp2->h/2;
      if (getpixel(bmp2, x, y)>0) { destroy_bitmap(bmp2); return rad*.8; }
      putpixel(screen, cos((ang)*pi/180)*25+320, sin((ang)*pi/180)*25+200, textcol);
    }
    rad--;
  }
  destroy_bitmap(bmp2);
  return rad;
}

class ent
{
public:
  float x, y, xvel, yvel;
  int angle, life, kill;
  int frame;
  int radius;
  sprite* sprit;

  void draw(BITMAP* scr, int cx, int cy);
  void drawsmall(BITMAP* scr, int cx, int cy);
};

/****************************************************************************/
/****************************************************************************/

class particle
{
public:
  particle *next;
  float x, y, xvel, yvel;
  char col, origcol;
  char life;
  int count;
  int kill;
  particleACTIONp action;

  void create(int xv, int yv, float xvelv, float yvelv, char startcol, particleACTIONp actionv)
  { x = xv; y = yv; xvel = xvelv; yvel = yvelv; action = actionv; count = 0; kill = 0; life = 60; origcol = startcol; }
};

/****************************************************************************/

class particleList
{
public:
  particle *base;
  void drawlist(BITMAP* scr, int cx, int cy);
  void drawlistsmall(BITMAP* scr, int cx, int cy);
  void movelist();
  void addParticle(int x, int y, float xvel, float yvel, char col, particleACTIONp actionv);
  void killOBJ(particle *kparticle);
  void killlist();
};

/****************************************************************************/

void particleList::movelist()
{
  particle *node = base;

  while (node != NULL)
  {
    node->action(node);
    if (node->kill) killOBJ(node);
    node = node->next;
  }
}

/****************************************************************************/

void particleList::killlist()
{
  particle *node = base;

  while (node != NULL)
  {
    killOBJ(node);
    node = node->next;
  }
}

/****************************************************************************/
/****************************************************************************/

void particleList::drawlist(BITMAP* scr, int cx, int cy)
{
  particle *node = base;

  while (node != NULL)
  {
    putpixel(scr, node->x-cx, node->y-cy, node->col);
    node = node->next;
  }
}

/****************************************************************************/
void particleList::drawlistsmall(BITMAP* scr, int cx, int cy)
{
  particle *node = base;

  while (node != NULL)
  {
    putpixel(scr, node->x/2, node->y/2, node->col);
    node = node->next;
  }
}

/****************************************************************************/

void particleList::addParticle(int x, int y, float xvel, float yvel, char col, particleACTIONp actionv)
{
  particle *node;

  node = new particle;

  node->create(x, y, xvel, yvel, 0, actionv);

  node->next = base;
  base = node;
}

/****************************************************************************/
void particleList::killOBJ(particle *kparticle)
{
  particle *node;

  node = new particle;

    if ( base == NULL )
        return;

    if ( base == kparticle )
    {
        base = kparticle->next;
        free( kparticle );
        return;
    }

  for (node = base; node->next != NULL; node = node->next)
    if ( node->next == kparticle )
    {
      node->next = kparticle->next;
      free( kparticle );
      return;
    }

  return;
}

//particleList particle_list;

/****************************************************************************/
/****************************************************************************/


/****************************************************************************/

void ent::draw(BITMAP* scr, int cx, int cy)
{
  sprit->draw(scr, x, y, cx, cy, angle, frame);
}

void ent::drawsmall(BITMAP* scr, int cx, int cy)
{
  sprit->drawsmall(scr, x, y, cx, cy, angle, frame);
}

/****************************************************************************/

class OBJ : public ent
{
public:
  OBJ *next;
  player *owner, *enemy;
  char count;
  ACTIONp action;

  void create(sprite* spr, int ax, int ay, float axvel, float ayvel, int ang);
  void move();
};

/****************************************************************************/

void OBJ::create(sprite* spr, int ax, int ay, float axvel, float ayvel, int ang)
{
  sprit = spr;
  x = ax;  y = ay;  xvel = axvel;  yvel = ayvel;  angle = ang; frame = 0;
}

/****************************************************************************/


/****************************************************************************/

void OBJ::move()
{
  x += xvel;  y += yvel;
}
/****************************************************************************/
/////////////////////////////////// Class Spritelist /////////////////////////
/****************************************************************************/
class OBJlist : public OBJ
{
public:
  OBJ *base;
  player *plr1, *plr2;
  void drawlist(BITMAP* scr, int cx, int cy);
  void movelist();
  void addOBJ(sprite *spr, int ax, int ay,
         float axvel, float ayvel, int ang, int alife, ACTIONp act, int owner);
  void spawnOBJ(sprite *spr, ACTIONp act, int owner);
  void killOBJ(OBJ *knode);
  void killlist();
};
/****************************************************************************/
void OBJlist::drawlist(BITMAP* scr, int cx, int cy)
{
  OBJ *node = base;

  if (mag)
  {
  while (node != NULL)
  {
    node->draw(scr, cx, cy);
    node = node->next;
  }
  }
  else
  {
  while (node != NULL)
  {
    node->drawsmall(scr, cx, cy);
    node = node->next;
  }
  }
}
/****************************************************************************/
void OBJlist::movelist()
{
  OBJ *node = base;

  while (node != NULL)
  {
    node->move();
    node->action(node);
    if (node->kill == 1) killOBJ(node);
    node = node->next;
  }
}
/****************************************************************************/
void OBJlist::killlist()
{
  OBJ *node = base;

  while (node != NULL)
  {
    killOBJ(node);
    node = node->next;
  }
}

/****************************************************************************/
void OBJlist::addOBJ(sprite *spr, int ax, int ay,
                           float axvel, float ayvel, int ang, int alife, ACTIONp act, int owner)
{
  OBJ *node;

  node = new OBJ;

  node->create(spr, ax, ay, axvel, ayvel, ang);
  node->x = ax;
  node->y = ay;
  node->xvel = axvel;
  node->yvel = ayvel;
  node->angle = ang;
  node->life = alife;
  node->action = act;
  node->kill = 0;
  if (owner == 0) { node->owner = plr1; node->enemy = plr2; }
  else { node->owner = plr2; node->enemy = plr1; }
  node->next = base;
  base = node;
}
/****************************************************************************/

void OBJlist::spawnOBJ(sprite *spr, ACTIONp act, int owner)
{
  int x, y, done=0;
  while (!done)
  {
    x = rand()%PLAYFIELDW;
    y = rand()%PLAYFIELDH;
    if (!map[(x/BLOCKSIZE)][(y/BLOCKSIZE)]) done = 1;
  }

  addOBJ(spr, x, y, 0, 0, 0, 0, act, 0);
}
/****************************************************************************/
void OBJlist::killOBJ(OBJ *knode)
{
  OBJ *node;

  node = new OBJ;

    if ( base == NULL )
    { return 0; }

    if ( base == knode )
    {
        base = knode->next;
        free( knode );
        return 1;
    }

  for (node = base; node->next != NULL; node = node->next)
    if ( node->next == knode )
    {
      node->next = knode->next;
      free( knode );
      return 1;
    }

  return 0;
}
/****************************************************************************/
////////////////////////////////// CLASS player //////////////////////////////
/****************************************************************************/

class player : public ent
{
public:
  int LEFT_KEY, RIGHT_KEY, THRUST_KEY, FIRE_KEY, SPECIAL_KEY, REVERSE_KEY, WPFW_KEY, WPBW_KEY;
  char count, number;
  int shotdelay, specialdelay, switchdelay;
  int money;
  char weapon, special;
  int ammo[7];
  int specammo[5];
  char select;
  char lives;
  char drw;
  float energy, life;
  PlayerACTIONp action;
  OBJlist *list;
  particleList *plist;

  void create(sprite* spr, int ax, int ay);
  void assign_inputs(int L_KEY, int R_KEY, int T_KEY, int F_KEY, int S_KEY, int RV_KEY);
};

/****************************************************************************/
void player::create(sprite* spr, int ax, int ay)
{
  sprit = spr;
  drw = 1;
  ammo[MACHINE_GUN] = -1;
  ammo[ROCKET_RACK] = 100;
  ammo[CANNON] = 25;
  ammo[NAPALM] = 50;
  ammo[LASER] = 0;
  ammo[BAZOOKA] = 0;
  ammo[NUKE] = 5;
  specammo[MINE] = 0;
  specammo[OIL] = 0;
  specammo[SHIELD] = 0;
  specammo[NONE] = -1;
  special = NONE; 
  money = 100000;
  lives = 3;
  x = ax;  y = ay;//  xvel = axvel;  yvel = ayvel;  angle = ang; frame = 0;
}
/****************************************************************************/

void player::assign_inputs(int L_KEY, int R_KEY, int T_KEY, int F_KEY, int S_KEY, int RV_KEY)
{
  LEFT_KEY = L_KEY;  RIGHT_KEY = R_KEY;  THRUST_KEY = T_KEY;
  FIRE_KEY = F_KEY;  SPECIAL_KEY = S_KEY; REVERSE_KEY = RV_KEY;
}

/****************************************************************************/
////////////////////////////////// Sprite Declarations ///////////////////////
/****************************************************************************/
sprite car1_spr, car2_spr, car3_spr, car4_spr;
sprite bazooka_spr, cannon_spr, laser_spr, rocketrack_spr, napalm_spr, nuke_spr, machinegun_spr;
sprite bazexp_spr, cannexp_spr, lasexp_spr, rrexp_spr, napexp_spr, nukeexp_spr, mineexp_spr, carexp_spr;
sprite mine_spr, oil_spr, shield_spr;
sprite fixup_spr, pitstop_spr, smallmoney_spr, bigmoney_spr, ammo_spr;


/****************************************************************************/
////////////////////////////////// PlayerACTION functions //////////////////////////
/****************************************************************************/
void truckgo(player *plr)
{
  plr->x += plr->xvel;
  plr->y += plr->yvel;
  plr->xvel *= 0.75;
  plr->yvel *= 0.75;

  if (plr->life < 100 && (plr->count%10)==0 && plr->specammo[BOT])
  {
    int ang, force, i;
    plr->specammo[BOT]--;
    plr->life++;

    for (i=0; i<15; i++)
    {
    ang = rand()%360;
    force = 0.3+(rand()%100)*.05;
    plr->plist->addParticle(plr->x, plr->y,
                            scale_x(ang)*(force),
                            scale_y(ang)*(force), 0, par_botsparkgo);
    }
  }

  int i;
  for (i=1; i<7; i++)
  {
    if (plr->ammo[i]<0 || plr->ammo[i]>255) plr->ammo[i]=255;
  }

  if (plr->switchdelay) plr->switchdelay--;
  if ((key[plr->WPFW_KEY] && !plr->switchdelay) || !plr->ammo[plr->weapon])
  {
  sw:
    plr->weapon++;
    if (plr->weapon>NUKE) plr->weapon = MACHINE_GUN;
    if (!plr->ammo[plr->weapon]) goto sw;
    plr->switchdelay = 10;
  }

    if (key[plr->WPBW_KEY] && !plr->switchdelay)
  {
  sw2:
    plr->weapon--;
    if (plr->weapon<MACHINE_GUN) plr->weapon = NUKE;
    if (!plr->ammo[plr->weapon]) goto sw2;
    plr->switchdelay = 10;
  }

    int mapchk=mapcolcheck(plr);
    if (mapchk)
    {
      if (mapchk&LEFT) plr->x = (int)(plr->x/32)*32+(plr->sprit->radius[plr->frame]+2);
      if (mapchk&RIGHT) plr->x = (int)(plr->x/32)*32+(plr->sprit->radius[plr->frame]-2);
      if (mapchk&TOP) plr->y = (int)(plr->y/32)*32+(plr->sprit->radius[plr->frame]+2);
      if (mapchk&BOTTOM) plr->y = (int)(plr->y/32)*32+(plr->sprit->radius[plr->frame]-2);
    }


  plr->count++;

  if (plr->shotdelay) plr->shotdelay--;
  if (plr->specialdelay) plr->specialdelay--;

  if (plr->x < 0) plr->x += PLAYFIELDW; if (plr->x > PLAYFIELDW) plr->x -= PLAYFIELDW;
  if (plr->y < 0) plr->y += PLAYFIELDH; if (plr->y > PLAYFIELDH) plr->y -= PLAYFIELDH;

  if (plr->energy < 40) plr->energy+=0.1;

  if (key[plr->LEFT_KEY]) plr->angle -= 2;
  if (key[plr->RIGHT_KEY]) plr->angle += 2;
  if (key[plr->THRUST_KEY])
  {
      plr->plist->addParticle(plr->x, plr->y,
                                    scale_x(plr->angle+180+rand()%30-15)*(.5),
                                    scale_y(plr->angle+180+rand()%30-15)*(.5), 0, par_ashgo);
    plr->xvel = scale_x(plr->angle)*2;
    plr->yvel = scale_y(plr->angle)*2;
  }
  if (key[plr->REVERSE_KEY])
  {
    plr->xvel = -scale_x(plr->angle)*1.5;
    plr->yvel = -scale_y(plr->angle)*1.5;
  }

  plr->frame = 0;

  if (key[plr->FIRE_KEY] && !plr->shotdelay)
  {
    plr->ammo[plr->weapon]--;
    switch (plr->weapon)
    {
    case MACHINE_GUN:
	plr->shotdelay = 5;
        plr->list->addOBJ(&machinegun_spr, plr->x, plr->y,
                          scale_x(plr->angle)*6, scale_y(plr->angle)*6, plr->angle, 0, machinegungo, plr->number);
	break;
    case ROCKET_RACK:
	plr->shotdelay = 15;
        plr->list->addOBJ(&rocketrack_spr, plr->x, plr->y,
                          scale_x(plr->angle+rand()%4-2)*4, scale_y(plr->angle+rand()%4-2)*4, plr->angle, 0, rocketrackgo, plr->number);
	break;
    case CANNON:
	plr->shotdelay = 50;
        plr->list->addOBJ(&cannon_spr, plr->x, plr->y,
                          scale_x(plr->angle)*3, scale_y(plr->angle)*3, plr->angle, 0, cannongo, plr->number);
        break;
    case BAZOOKA:
	plr->shotdelay = 75;
        plr->list->addOBJ(&bazooka_spr, plr->x, plr->y,
                          scale_x(plr->angle)*4, scale_y(plr->angle)*4, plr->angle, 0, bazookago, plr->number);
        break;
    case NUKE:
	plr->shotdelay = 100;
        plr->list->addOBJ(&nuke_spr, plr->x, plr->y,
                          scale_x(plr->angle)*2, scale_y(plr->angle)*2, plr->angle, 0, nukego, plr->number);
        break;
    case LASER:
	plr->shotdelay = 50;
        plr->list->addOBJ(&laser_spr, plr->x, plr->y,
                          scale_x(plr->angle)*12, scale_y(plr->angle)*12, plr->angle, 0, laser_spawnergo, plr->number);
        break;
    case NAPALM:
	plr->shotdelay = 10;
        plr->list->addOBJ(&napalm_spr, plr->x, plr->y,
                          scale_x(plr->angle)*5, scale_y(plr->angle)*5, plr->angle, 0, napalmgo, plr->number);
        break;
    }
  }

  if (key[plr->SPECIAL_KEY] && !plr->specialdelay)
  {
    switch (plr->special)
    {
    case MINE:
        plr->specammo[MINE]--;
        plr->specialdelay = 40;
        plr->list->addOBJ(&mine_spr, plr->x, plr->y,
                          0, 0, 0, 0, minego, plr->number);
        break;
    case OIL:
	  plr->specammo[OIL]--;
        plr->specialdelay = 50;
        plr->list->addOBJ(&oil_spr, plr->x, plr->y,
                          0, 0, 0, 0, oilgo, plr->number);
        break;
    case SHIELD:
	  plr->specammo[SHIELD]--;
        plr->specialdelay = 200;
        plr->list->addOBJ(&shield_spr, plr->x, plr->y,
                          0, 0, 0, 0, shieldgo, plr->number);
        break;
  }
  
  if (plr->specammo[plr->special]==0)
  {
    while (plr->specammo[plr->special]==0)
    {
      plr->special++;
      if (plr->special>BOT);
      plr->special = NONE;
    }
  }
}

  if (plr->life < 0) plr->list->addOBJ(&carexp_spr, plr->x, plr->y,
                          0, 0, 0, 0, carexpgo, plr->number);
}

void truckspingo(player *plr)
{
  if (plr->count==0)
  {
    int ang;
    ang = plr->angle+(rand()%60-30);
    plr->xvel = scale_x(ang)*5;
    plr->yvel = scale_y(ang)*5;
  }
  plr->count++;

  plr->angle+=8*(100-plr->count)/75;
  plr->angle%=360;

  plr->x+=plr->xvel;
  plr->y+=plr->yvel;

    int mapchk=mapcolcheck(plr);
    if (mapchk)
    {
      if (mapchk&LEFT) plr->x = (int)(plr->x/32)*32+(plr->sprit->radius[plr->frame]+2);
      if (mapchk&RIGHT) plr->x = (int)(plr->x/32)*32+(plr->sprit->radius[plr->frame]-2);
      if (mapchk&TOP) plr->y = (int)(plr->y/32)*32+(plr->sprit->radius[plr->frame]+2);
      if (mapchk&BOTTOM) plr->y = (int)(plr->y/32)*32+(plr->sprit->radius[plr->frame]-2);
    }

  if (plr->count==100) plr->action = truckgo;
}
/****************************************************************************/
void deadtruckgo(player *plr)
{
  plr->life = 0;
  plr->x = -100;
  plr->y = -100;
  mag = 0;
  if (plr->count>100)
  {
    plr->life = 100;
    plr->lives--;
    plr->drw = 1;
    int done = 0;
    while (!done)
    {
      plr->x = rand()%PLAYFIELDW;
      plr->y = rand()%PLAYFIELDH;
      if (!mapcolcheck(plr)) done = 1;
    }
    plr->action = truckgo;
  }
  plr->count++;
}

/****************************************************************************/
void shoptruckgo(player *plr)
{
  if (plr->count==0)
  {
    plr->x -= 1000;
    plr->y -= 1000;
  }
  mag = 0;

  if (plr->number==0)
  {
  textprintf_centre(scrbuf, font, 288, 10, 80, "Repair Job - $%d", (100-plr->life)*200);
  textprintf_centre(scrbuf, font, 288, 20, 80, "Repair Bot - $50000");
  textprintf_centre(scrbuf, font, 288, 30, 80, "Mines (25) - $50000");
  textprintf_centre(scrbuf, font, 288, 40, 80, "Shield (5) - $50000");
  textprintf_centre(scrbuf, font, 288, 50, 80, "Oil Slick (10) - $10000");
  textprintf_centre(scrbuf, font, 288, 60, 80, "Cannon Shells (20) - $10000");
  textprintf_centre(scrbuf, font, 288, 70, 80, "Rockets (50) - $10000");
  textprintf_centre(scrbuf, font, 288, 80, 80, "Napalm (50) - $25000");
  textprintf_centre(scrbuf, font, 288, 90, 80, "Laser Batteries (10) - $30000");
  textprintf_centre(scrbuf, font, 288, 100, 80, "Bazooka Rounds (10) - $30000");
  textprintf_centre(scrbuf, font, 288, 110, 80, "Nuke (1) - $5000");
  textprintf_centre(scrbuf, font, 288, 120, 80, "Exit Shop");

  switch(plr->select)
  {
  case 0:
  textprintf_centre(scrbuf, font, 288, 10, 90, "Repair Job - $%d", (int)(100-plr->life)*200);
  break;
  case 1:
  textprintf_centre(scrbuf, font, 288, 20, 90, "Repair Bot - $50000");
  break;
  case 2:
  textprintf_centre(scrbuf, font, 288, 30, 90, "Mines (25) - $50000");
  break;
  case 3:
  textprintf_centre(scrbuf, font, 288, 40, 90, "Shield (5) - $50000");
  break;
  case 4:
  textprintf_centre(scrbuf, font, 288, 50, 90, "Oil Slick (10) - $10000");
  break;
  case 5:
  textprintf_centre(scrbuf, font, 288, 60, 90, "Cannon Shells (20) - $10000");
  break;
  case 6:
  textprintf_centre(scrbuf, font, 288, 70, 90, "Rockets (50) - $10000");
  break;
  case 7:
  textprintf_centre(scrbuf, font, 288, 80, 90, "Napalm (50) - $25000");
  break;
  case 8:
  textprintf_centre(scrbuf, font, 288, 90, 90, "Laser Batteries (10) - $30000");
  break;
  case 9:
  textprintf_centre(scrbuf, font, 288, 100, 90, "Bazooka Rounds (10) - $30000");
  break;
  case 10:
  textprintf_centre(scrbuf, font, 288, 110, 90, "Nuke (1) - $5000");
  break;
  case 11:
  textprintf_centre(scrbuf, font, 288, 120, 90, "Exit Shop");
  break;
  }

  }
  else
  {
  textprintf_centre(scrbuf, font, 288, 300, 80, "Repair Job - $%d", (100-plr->life)*200);
  textprintf_centre(scrbuf, font, 288, 310, 80, "Repair Bot - $50000");
  textprintf_centre(scrbuf, font, 288, 320, 80, "Mines (25) - $50000");
  textprintf_centre(scrbuf, font, 288, 330, 80, "Shield (5) - $50000");
  textprintf_centre(scrbuf, font, 288, 340, 80, "Oil Slick (10) - $10000");
  textprintf_centre(scrbuf, font, 288, 350, 80, "Cannon Shells (20) - $10000");
  textprintf_centre(scrbuf, font, 288, 360, 80, "Rockets (50) - $10000");
  textprintf_centre(scrbuf, font, 288, 370, 80, "Napalm (50) - $25000");
  textprintf_centre(scrbuf, font, 288, 380, 80, "Laser Batteries (10) - $30000");
  textprintf_centre(scrbuf, font, 288, 390, 80, "Bazooka Rounds (10) - $30000");
  textprintf_centre(scrbuf, font, 288, 400, 80, "Nuke (1) - $5000");
  textprintf_centre(scrbuf, font, 288, 410, 80, "Exit Shop");

  switch(plr->select)
  {
  case 0:
  textprintf_centre(scrbuf, font, 288, 300, 90, "Repair Job - $%d", (int)(100-plr->life)*200);
  break;
  case 1:
  textprintf_centre(scrbuf, font, 288, 310, 90, "Repair Bot - $50000");
  break;
  case 2:
  textprintf_centre(scrbuf, font, 288, 320, 90, "Mines (25) - $50000");
  break;
  case 3:
  textprintf_centre(scrbuf, font, 288, 330, 90, "Shield (5) - $50000");
  break;
  case 4:
  textprintf_centre(scrbuf, font, 288, 340, 90, "Oil Slick (10) - $10000");
  break;
  case 5:
  textprintf_centre(scrbuf, font, 288, 350, 90, "Cannon Shells (20) - $10000");
  break;
  case 6:
  textprintf_centre(scrbuf, font, 288, 360, 90, "Rockets (50) - $10000");
  break;
  case 7:
  textprintf_centre(scrbuf, font, 288, 370, 90, "Napalm (50) - $25000");
  break;
  case 8:
  textprintf_centre(scrbuf, font, 288, 380, 90, "Laser Batteries (10) - $30000");
  break;
  case 9:
  textprintf_centre(scrbuf, font, 288, 390, 90, "Bazooka Rounds (10) - $30000");
  break;
  case 10:
  textprintf_centre(scrbuf, font, 288, 400, 90, "Nuke (1) - $5000");
  break;
  case 11:
  textprintf_centre(scrbuf, font, 288, 410, 90, "Exit Shop");
  break;
  }
  }

  if (key[plr->REVERSE_KEY]&&plr->count%10==0) plr->select++;
  if (key[plr->THRUST_KEY]&&plr->count%10==0) plr->select--;

  plr->select = (plr->select+12)%12;

  if (key[plr->FIRE_KEY]&&plr->count%10==0)
  {
  switch(plr->select)
  {
  case 0:
    if (plr->money>=(100-plr->life)*200)
    {
    plr->money -= (100-plr->life)*200;
    plr->life = 100;
    }
  break;
  case 1:
    if (plr->money>=50000)
    {
      plr->money -= 50000;
      plr->specammo[BOT]+=100;
      plr->special = BOT;
    rectfill(screen, 16+688*plr->number, 168, 45+688*plr->number, 193, 0);
    }
  break;
  case 2:
    if (plr->money>=50000)
    {
      plr->money -= 50000;
      plr->specammo[MINE] += 25;
      plr->special = MINE;
    rectfill(screen, 16+688*plr->number, 168, 45+688*plr->number, 193, 0);
    }
  break;
  case 3:
    if (plr->money>=50000)
    {
    plr->money -= 50000;
    plr->specammo[SHIELD] += 5;
    plr->special = SHIELD;
    rectfill(screen, 16+688*plr->number, 168, 45+688*plr->number, 193, 0);
    }
  break;
  case 4:
    if (plr->money>=10000)
    {
    plr->money -= 10000;
    plr->specammo[OIL] += 10;
    plr->special = OIL;
    rectfill(screen, 16+688*plr->number, 168, 45+688*plr->number, 193, 0);
    }
  break;
  case 5:
    if (plr->money>=10000)
    {
    plr->money -= 10000;
    plr->ammo[CANNON] += 10;
    }
  break;
  case 6:
    if (plr->money>=10000)
    {
    plr->money -= 10000;
    plr->ammo[ROCKET_RACK] += 50;
    }
  break;
  case 7:
    if (plr->money>=25000)
    {
    plr->money -= 25000;
    plr->ammo[NAPALM] += 50;
    }
  break;
  case 8:
    if (plr->money>=30000)
    {
    plr->money -= 30000;
    plr->ammo[LASER] += 10;
    }
  break;
  case 9:
    if (plr->money>=30000)
    {
    plr->money -= 30000;
    plr->ammo[BAZOOKA] += 10;
    }
  break;
  case 10:
    if (plr->money>=5000)
    {
    plr->money -= 5000;
    plr->ammo[NUKE]++;
    }
  break;
  case 11:
    plr->drw = 1;
    int done = 0;
    while (!done)
    {
      plr->x = rand()%PLAYFIELDW;
      plr->y = rand()%PLAYFIELDH;
      if (!mapcolcheck(plr)) done = 1;
    }
    plr->action = truckgo;
  break;
  }
  }

  if (plr->money<1)
  {
    plr->drw = 1;
    int done = 0;
    while (!done)
    {
      plr->x = rand()%PLAYFIELDW;
      plr->y = rand()%PLAYFIELDH;
      if (!mapcolcheck(plr)) done = 1;
    }
    plr->action = truckgo;
  }

  plr->count++;
}
/****************************************************************************/

/****************************************************************************/
////////////////////////////////// ACTION functions //////////////////////////
/****************************************************************************/

/****************************************************************************/
void carexpgo(OBJ *obj)
{
  obj->x = obj->owner->x;
  obj->y = obj->owner->y;

  obj->owner->life = 0;

  if (obj->life > 25) obj->owner->drw = 0;

  if (obj->life > 10) { obj->frame = 1; }
  if (obj->life > 15) { obj->frame = 2; }
  if (obj->life > 25) { obj->frame = 3; }
  if (obj->life > 30) { obj->frame = 4; }
  if (obj->life > 35) { obj->frame = 5; }
  if (obj->life > 40) { obj->kill = 1; obj->owner->action = deadtruckgo; obj->owner->count = 0; }

  obj->life++;
}

void machinegungo(OBJ *obj)
{
  int i;

  if (obj->life%10==0)
        obj->owner->plist->addParticle(obj->x, obj->y,
                                    obj->xvel,
                                    obj->yvel, 0, par_bulletgo);


  if (mapcolcheck(obj))
  {
    for (i=0; i<4; i++)
    {
      int ang;
      float force;
      ang = rand()%360;
      force = 0.3+(rand()%100)*.05;
      obj->owner->plist->addParticle(obj->x, obj->y,
                                    scale_x(ang)*(force),
                                    scale_y(ang)*(force), 0, par_ashgo);
    }
    obj->kill = 1;
  }

  if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= .5;

    for (i=0; i<4; i++)
    {
      int ang;
      float force;
      ang = rand()%360;
      force = 0.3+(rand()%100)*.05;
      obj->owner->plist->addParticle(obj->x, obj->y,
                                    scale_x(ang)*(force),
                                    scale_y(ang)*(force), 0, par_ashgo);
    }
    obj->kill = 1;
  }

  obj->life++;
}

/****************************************************************************/

void rocketrackgo(OBJ *obj)
{

 if(obj->life%2==0)
 {
   obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_flamego);
 }

 if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&rrexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, rrexpgo, obj->owner->number);
  }

 if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 3;

    obj->kill = 1;
    obj->owner->list->addOBJ(&rrexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, rrexpgo, obj->owner->number);
  }
  obj->life++;

  if (obj->life%20==0) obj->frame ^= 1;
}

/****************************************************************************/

void rrexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 25)
  for (i=0; i<10; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.05;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_ashgo);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 2; }
  if (obj->life > 25) { obj->frame = 1; }
  if (obj->life > 30) { obj->frame = 0; }
  if (obj->life > 35) { obj->kill = 1; }

  obj->life++;
}

/****************************************************************************/

void cannongo(OBJ *obj)
{

 if(obj->life%4==0)
 {
   obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_ashgo);
 }

 if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&cannexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, cannexpgo, obj->owner->number);
  }
 if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 8;
    obj->kill = 1;
    obj->owner->list->addOBJ(&cannexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, cannexpgo, obj->owner->number);
  }
  obj->life++;
}

/****************************************************************************/

void napalmgo(OBJ *obj)
{

 if(obj->life%4==0)
 {
   obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_flamego);
 }

 if (obj->life > 40+rand()%20)
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&napexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle+rand()%60-30, 0, napexpgo, obj->owner->number);
  }

  if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&napexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, napexpgo, obj->owner->number);
  }
 if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 5;
    obj->kill = 1;
    obj->owner->list->addOBJ(&napexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, napexpgo, obj->owner->number);
  }

  if (obj->life%25==0) obj->frame ^= 1;

  obj->life++;
}

/****************************************************************************/

void napexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 5)
  for (i=0; i<50; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.025;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_flamego);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 2; }
  if (obj->life > 25) { obj->frame = 3; }
  if (obj->life > 30) { obj->frame = 2; }
  if (obj->life > 35) { obj->frame = 1; }
  if (obj->life > 40) { obj->frame = 0; }
  if (obj->life > 45) { obj->kill = 1; }

 if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= .75;
    obj->life += 5;
  }
  obj->life++;
}

/****************************************************************************/
/****************************************************************************/

void nukego(OBJ *obj)
{

 obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_flamego);
 obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_flamego);

  if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&nukeexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, nukeexpgo, obj->owner->number);
  }

  if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 40;
    obj->kill = 1;
    obj->owner->list->addOBJ(&nukeexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, nukeexpgo, obj->owner->number);
  }
  obj->life++;
}

/****************************************************************************/

void nukesmokego(OBJ *obj)
{
  int i;
  if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 2;
    obj->kill = 1;
    obj->owner->list->addOBJ(&rrexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, rrexpgo, obj->owner->number);
  }

  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 2; }
  if (obj->life > 25) { obj->frame = 1; }
  if (obj->life > 30) { obj->frame = 0; }
  if (obj->life > 35) { obj->kill = 1; }

  obj->life++;
}

/****************************************************************************/

void nukeexpgo(OBJ *obj)
{
  int i;
  if (obj->life > 10)
  {
      int ang;
      float force;
      ang = rand()%360;
      force = rand()%100 - 50;
      obj->owner->list->addOBJ(&rrexp_spr, obj->x+scale_x(ang)*force, obj->y+scale_y(ang)*force,
                               0, 0, obj->angle, 0, nukesmokego, obj->owner->number);
  }
  if (obj->life == 25)
  {
    for (i=0; i<150; i++)
    {
      int ang;
      float force;
      ang = rand()%360;
      force = 0.3+(rand()%100)*.05;
      obj->owner->plist->addParticle(obj->x, obj->y,
                                    scale_x(ang)*(force),
                                    scale_y(ang)*(force), 0, par_flamego);
    }
    for (i=0; i<360; i+=40)
    {
      obj->owner->list->addOBJ(&rrexp_spr, obj->x+scale_x(i)*10, obj->y+scale_y(i)*10,
                               scale_x(i)*1, scale_y(i)*1, obj->angle, 0, nukesmokego, obj->owner->number);
    }
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 4; }
  if (obj->life > 25) { obj->kill = 1; }

  obj->life++;
}

/****************************************************************************/
/****************************************************************************/

void laser_spawnergo(OBJ *obj)
{
 if (obj->life==0)
 {
   int i;
   for (i=0; i<10; i++)
   {
     obj->owner->list->addOBJ(&laser_spr, obj->x, obj->y,
                               0, 0, obj->angle, -10+i, lasergo, obj->owner->number);
   }
 }

  if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&lasexp_spr, obj->x, obj->y,
                             obj->enemy->xvel, obj->enemy->yvel, obj->angle, 0, lasexpgo, obj->owner->number);
  }
 if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 5;
    obj->kill = 1;
    obj->owner->list->addOBJ(&lasexp_spr, obj->x, obj->y,
                             obj->enemy->xvel, obj->enemy->yvel, obj->angle, 0, lasexpgo, obj->owner->number);
  }
  obj->life++;
}

/****************************************************************************/
void lasergo(OBJ *obj)
{
 if (obj->life>=0)
 {
   obj->xvel = scale_x(obj->angle)*12;
   obj->yvel = scale_y(obj->angle)*12;
 }

 if(obj->life%2==0)
 {
   obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_lasergo);
 }

  if (mapcolcheck(obj))
  {
    obj->kill = 1;
  }

  if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 2;
    obj->kill = 1;
  }
  obj->life++;
}
/****************************************************************************/

void lasexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 10)
  for (i=0; i<50; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.025;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_lasergo);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 1; }
  if (obj->life > 20) { obj->frame = 0; }
  if (obj->life > 25) { obj->kill = 1; }

  obj->life++;
}

/****************************************************************************/

void shieldgo(OBJ *obj)
{
  int i;
  if (obj->life==0)
  {
    obj->count = obj->owner->life;
    for (i=0; i<360; i+=30)
    {
      obj->owner->list->addOBJ(&shield_spr, obj->x, obj->y,
                               0, 0, obj->angle, 360-i, shieldballgo, obj->owner->number);
    }
  }
  if (obj->owner->life < obj->count)
  {
    int i;
    for (i=0; i<(obj->count-obj->owner->life)*8; i++)
    {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.025;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_shieldsparkgo);
    }
    obj->owner->life = obj->count;
  }

  obj->x = obj->owner->x + scale_x(obj->life)*20;
  obj->y = obj->owner->y + scale_y(obj->life)*20;

  obj->xvel = obj->owner->xvel;
  obj->yvel = obj->owner->yvel;

  obj->life+=4;

  if (obj->life > 720) obj->kill = 1;
}

/****************************************************************************/

void shieldballgo(OBJ *obj)
{
  obj->x = obj->owner->x + scale_x(obj->life)*20;
  obj->y = obj->owner->y + scale_y(obj->life)*20;

  obj->xvel = obj->owner->xvel;
  obj->yvel = obj->owner->yvel;

  obj->life+=4;

  if (obj->life > 720) obj->kill = 1;
}

/****************************************************************************/

/****************************************************************************/

void bazookago(OBJ *obj)
{

 if(obj->life%2==0)
 {
   obj->owner->plist->addParticle(obj->x-scale_x(obj->angle)*8, obj->y-scale_y(obj->angle)*8,
                                  scale_x(obj->angle+rand()%40-20)*0.5,
                                  scale_y(obj->angle+rand()%40-20)*0.5, 0, par_flamego);
 }


  if (mapcolcheck(obj))
  {
    obj->kill = 1;
    obj->owner->list->addOBJ(&bazexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, bazexpgo, obj->owner->number);
  }

  if (colcheck(obj, obj->enemy)==1)
  {
    obj->enemy->life -= 20;
    obj->kill = 1;
    obj->owner->list->addOBJ(&bazexp_spr, obj->x, obj->y,
                             obj->enemy->xvel+obj->xvel/10, obj->enemy->yvel+obj->yvel/10, obj->angle, 0, bazexpgo, obj->owner->number);
  }
  obj->life++;
}

/****************************************************************************/
/****************************************************************************/
void bazexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 5)
  for (i=0; i<200; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.05;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_flamego);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 4; }
  if (obj->life > 25) { obj->frame = 3; }
  if (obj->life > 30) { obj->frame = 2; }
  if (obj->life > 35) { obj->frame = 1; }
  if (obj->life > 40) { obj->frame = 0; }
  if (obj->life > 45) { obj->kill = 1; }

  obj->life++;
}
/****************************************************************************/
/****************************************************************************/
void cannexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 10)
  for (i=0; i<100; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.05;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_ashgo);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 20) { obj->frame = 4; }
  if (obj->life > 25) { obj->frame = 3; }
  if (obj->life > 30) { obj->frame = 2; }
  if (obj->life > 35) { obj->frame = 1; }
  if (obj->life > 40) { obj->frame = 0; }
  if (obj->life > 45) { obj->kill = 1; }

  obj->life++;
}
/****************************************************************************/

void minego(OBJ *obj)
{
 if (obj->life%18==0 && obj->life > 25) obj->frame ^= 1;

 if ((colcheck(obj, obj->enemy)==1) && obj->frame==1)
 {
   obj->enemy->life -= 10;
    obj->kill = 1;
    obj->owner->list->addOBJ(&mineexp_spr, obj->x, obj->y,
                             0, 0, obj->angle, 0, mineexpgo, obj->owner->number);
  }
  obj->life++;
}

/****************************************************************************/
/****************************************************************************/
void mineexpgo(OBJ *obj)
{
  int i;
  if (obj->life == 5)
  for (i=0; i<200; i++)
  {
    int ang;
    float force;
    ang = rand()%360;
    force = 0.3+(rand()%100)*.05;
    obj->owner->plist->addParticle(obj->x, obj->y,
                                  scale_x(ang)*(force),
                                  scale_y(ang)*(force), 0, par_flamego);
  }
  if (obj->life > 5) { obj->frame = 1; }
  if (obj->life > 10) { obj->frame = 2; }
  if (obj->life > 15) { obj->frame = 3; }
  if (obj->life > 18) { obj->frame = 2; }
  if (obj->life > 21) { obj->frame = 1; }
  if (obj->life > 24) { obj->frame = 0; }
  if (obj->life > 28) { obj->kill = 1; }

  obj->life++;
}
/****************************************************************************/

void oilgo(OBJ *obj)
{
 if (colcheck(obj, obj->enemy)==1 && obj->life>10)
 {
    obj->kill = 1;
    obj->enemy->count = 0;
    obj->enemy->action = truckspingo;
 }
 if (colcheck(obj, obj->owner)==1 && obj->life>50)
 {
    obj->kill = 1;
    obj->owner->count = 0;
    obj->owner->action = truckspingo;
 }
  obj->life++;
}
/****************************************************************************/

void pitstopgo(OBJ *obj)
{
 if (colcheck(obj, obj->enemy)==1)
 {
    obj->enemy->count = 0;
    obj->enemy->action = shoptruckgo;
 }
 if (colcheck(obj, obj->owner)==1)
 {
    obj->owner->count = 0;
    obj->owner->action = shoptruckgo;
 }
  obj->life++;
}

/****************************************************************************/

void fixupgo(OBJ *obj)
{
 if (colcheck(obj, obj->enemy)==1)
 {
   obj->enemy->life+=50; if (obj->enemy->life > 100) obj->enemy->life = 100;
   obj->kill = 1;
 }
 if (colcheck(obj, obj->owner)==1)
 {
   obj->owner->life+=50; if (obj->owner->life > 100) obj->owner->life = 100;
   obj->kill = 1;
 }
  obj->life++;
}

/****************************************************************************/

void smallmoneygo(OBJ *obj)
{
 if (colcheck(obj, obj->enemy)==1)
 {
   obj->enemy->money += 5000;
   obj->kill = 1;
 }
 if (colcheck(obj, obj->owner)==1)
 {
   obj->owner->money += 5000;
   obj->kill = 1;
 }
  obj->life++;
}
/****************************************************************************/

void bigmoneygo(OBJ *obj)
{
 if (colcheck(obj, obj->enemy)==1)
 {
   obj->enemy->money += 20000;
   obj->kill = 1;
 }
 if (colcheck(obj, obj->owner)==1)
 {
   obj->owner->money += 20000;
   obj->kill = 1;
 }
  obj->life++;
}

/****************************************************************************/

void ammogo(OBJ *obj)
{
 int wep = rand()%6+1;

 if (colcheck(obj, obj->enemy)==1)
 {
   switch (wep)
   {
   case ROCKET_RACK:
     obj->enemy->ammo[ROCKET_RACK] += 50;
     break;
   case CANNON:
     obj->enemy->ammo[CANNON] += 20;
     break;
   case NAPALM:
     obj->enemy->ammo[NAPALM] += 50;
     break;
   case LASER:
     obj->enemy->ammo[LASER] += 15;
     break;
   case BAZOOKA:
     obj->enemy->ammo[BAZOOKA] += 10;
     break;
   case NUKE:
     obj->enemy->ammo[NUKE] += 3;
     break;
   }
   if (obj->enemy->ammo[wep]>255||obj->enemy->ammo[wep]<0) obj->enemy->ammo[wep] = 255;
   obj->kill = 1;
 }
 if (colcheck(obj, obj->owner)==1)
 {
   switch (wep)
   {
   case ROCKET_RACK:
     obj->owner->ammo[ROCKET_RACK] += 50;
     break;
   case CANNON:
     obj->owner->ammo[CANNON] += 20;
     break;
   case NAPALM:
     obj->owner->ammo[NAPALM] += 50;
     break;
   case LASER:
     obj->owner->ammo[LASER] += 15;
     break;
   case BAZOOKA:
     obj->owner->ammo[BAZOOKA] += 10;
     break;
   case NUKE:
     obj->owner->ammo[NUKE] += 3;
     break;
   }
   if (obj->owner->ammo[wep]>255||obj->owner->ammo[wep]<0) obj->owner->ammo[wep] = 255;
   obj->kill = 1;
 }
  obj->life++;
}

/****************************************************************************/
/****************************************************************************/

/****************************************************************************/
/***************************** Begin Particle Action Functions **************/
/****************************************************************************/
/****************************************************************************/
void par_flamego (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 94;
  if (objp->count>10) objp->col = 14;
  if (objp->count>20) objp->col = 133;
  if (objp->count>30) objp->col = 168;
  if (objp->count>40) objp->col = 165;
  if (objp->count>50) objp->col = 162;
  if (objp->count>60) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
void par_ashgo (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 92;
  if (objp->count>10) objp->col = 91;
  if (objp->count>15) objp->col = 90;
  if (objp->count>20) objp->col = 89;
  if (objp->count>30) objp->col = 87;
  if (objp->count>40) objp->col = 85;
  if (objp->count>50) objp->col = 83;
  if (objp->count>60) objp->col = 81;
  if (objp->count>70) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
void par_bulletgo (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 0;
  if (objp->count>10) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
void par_lasergo (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 63;
  if (objp->count>10) objp->col = 61;
  if (objp->count>15) objp->col = 59;
  if (objp->count>20) objp->col = 57;
  if (objp->count>30) objp->col = 55;
  if (objp->count>40) objp->col = 53;
  if (objp->count>50) objp->col = 52;
  if (objp->count>60) objp->col = 50;
  if (objp->count>70) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
void par_shieldsparkgo (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 30;
  if (objp->count>10) objp->col = 29;
  if (objp->count>15) objp->col = 28;
  if (objp->count>20) objp->col = 26;
  if (objp->count>30) objp->col = 24;
  if (objp->count>40) objp->col = 22;
  if (objp->count>50) objp->col = 20;
  if (objp->count>60) objp->col = 18;
  if (objp->count>70) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
void par_botsparkgo (particle *objp)
{
  int ang;
  int col, r, g, b;

  if (objp->count==0) objp->col = 94;
  if (objp->count>10) objp->col = 9;
  if (objp->count>25) objp->col = 79;
  if (objp->count>30) objp->col = 78;
  if (objp->count>40) objp->col = 77;
  if (objp->count>45) objp->col = 74;
  if (objp->count>50) objp->col = 71;
  if (objp->count>60) objp->kill = 1;

  objp->count++;

  objp->x += objp->xvel;
  objp->y += objp->yvel;

}
/****************************************************************************/
/***************************** End Particle Action Functions ****************/
/****************************************************************************/

/****************************************************************************/
/////////////////////////////////// BODY /////////////////////////////////////
/****************************************************************************/

int i, i2;
int cx = 0, cy = 0, tx, ty;
int xn1, xn2, yn1, yn2;
float xv1, yv1;

main()
{
  init_system();

  do_menu();

  printf("Thanks for playing!\nEben Olson, baniel@crosswinds.net");
}
/****************************************************************************/
////////////////////////////////// Main Functions/////////////////////////////
/****************************************************************************/
  player plr1;
  player plr2;
  OBJlist list1;
  particleList plist1;

void playgame(int car1, int car2, int map)
{
  int car1x, car2x, car1y, car2y;
  done = 0;
  mag = 0;

  clear(screen);
  clear(scrbuf);

  switch (car1)
  {
    case 0:
      plr1.create(&car1_spr, 0, 0);
      break;
    case 1:
      plr1.create(&car2_spr, 0, 0);
      break;
    case 2:
      plr1.create(&car3_spr, 0, 0);
      break;
    case 3:
      plr1.create(&car4_spr, 0, 0);
      break;
  }
  plr1.assign_inputs(KEY_A, KEY_D, KEY_W, KEY_F, KEY_G, KEY_S);
  plr1.list = &list1;
  plr1.plist = &plist1;
  plr1.action = truckgo;
  plr1.frame = 0;
  plr1.life = 100;
  plr1.number = 0;
  plr1.WPFW_KEY = KEY_CAPSLOCK;
  plr1.WPBW_KEY = KEY_LSHIFT;

  switch (car2)
  {
    case 0:
      plr2.create(&car1_spr, 0, 0);
      break;
    case 1:
      plr2.create(&car2_spr, 0, 0);
      break;
    case 2:
      plr2.create(&car3_spr, 0, 0);
      break;
    case 3:
      plr2.create(&car4_spr, 0, 0);
      break;
  }
  plr2.assign_inputs(KEY_DEL, KEY_PGDN, KEY_HOME, KEY_NUMLOCK, KEY_SLASH2, KEY_END);
  plr2.action = truckgo;
  plr2.list = &list1;
  plr2.plist = &plist1;
  plr2.frame = 0;
  plr2.life = 100;
  plr2.number = 1;
  plr2.WPFW_KEY = KEY_BACKSPACE;
  plr2.WPBW_KEY = KEY_BACKSLASH;

  list1.plr1 = &plr1;
  list1.plr2 = &plr2;

  switch(map)
  {
  case 0:
    backg = load_bmp("./images/backgrnd/bg.bmp", pal);
    stretch_blit(backg, lowrezbg, 0, 0, 1152, 1152, 0, 0, 576, 576);
    load_level("bts1.map");
    list1.addOBJ(&pitstop_spr, 280, 350, 0, 0, 0, 0, pitstopgo, 0);
    list1.addOBJ(&pitstop_spr, 870, 800, 0, 0, 0, 0, pitstopgo, 0);
  break;
  }

  while (!done)
  {
    plr1.x = rand()%PLAYFIELDW;
    plr1.y = rand()%PLAYFIELDH;
    if (!mapcolcheck(&plr1)) done = 1;
  }
  done = 0;
  while (!done)
  {
    plr2.x = rand()%PLAYFIELDW;
    plr2.y = rand()%PLAYFIELDH;
    if (!mapcolcheck(&plr2)) done = 1;
  }
  done = 0;

  draw_sprite(screen, panel[car1], 0, 0);
  draw_sprite(screen, panel[car2], 688, 0);

while (!done)
{
xn1 = plr1.x; xn2 = plr2.x;
yn1 = plr1.y; yn2 = plr2.y;

if (nomag) mag = 0;
if (mag)
{
  cx = (xn1+xn2-576)/2;
  cy = (yn1+yn2-576)/2;
  if (cx<0) cx = 0; if (cx+576>PLAYFIELDW) cx = PLAYFIELDW-576;
  if (cy<0) cy = 0; if (cy+576>PLAYFIELDH) cy = PLAYFIELDH-576;
      blit(backg, scrbuf, cx, cy, 0, 0, 576, 576);
      if (plr1.drw) plr1.draw(scrbuf, cx, cy);
      if (plr2.drw) plr2.draw(scrbuf, cx, cy);
      list1.drawlist(scrbuf, cx, cy);
      plist1.drawlist(scrbuf, cx, cy);
}
else
{
  cx = 0;
  cy = 0;
      blit(lowrezbg, scrbuf, 0, 0, 0, 0, 576, 576);
      plr1.drawsmall(scrbuf, cx, cy);
      plr2.drawsmall(scrbuf, cx, cy);
      list1.drawlist(scrbuf, cx, cy);
      plist1.drawlistsmall(scrbuf, cx, cy);
}

if (abs(xn1-xn2)<576 && abs(yn1-yn2)<576) mag = 1;
else mag = 0;

    textprintf_centre(scrbuf, font, 320, 20, 61, "FPS: %d", fps);
logic:
    if (gologic)
    {
      if (rand()%moneyrate==1) list1.spawnOBJ(&smallmoney_spr, smallmoneygo, 0);
      if (rand()%(moneyrate*5)==1 && moneyrate > 0) list1.spawnOBJ(&bigmoney_spr, bigmoneygo, 0);
      if (rand()%fixuprate==1) list1.spawnOBJ(&fixup_spr, fixupgo, 0);
      if (rand()%ammorate==1) list1.spawnOBJ(&ammo_spr, ammogo, 0);
      gologic = 0;
      plr1.action(&plr1);
      plr2.action(&plr2);
      list1.movelist();
      plist1.movelist();
    }
    if (gologic) goto logic;

    update_panels(plr1, plr2);

//    if (count%10==0) palrot();

    blit(scrbuf, screen, 0, 0, 112, 12, 576, 576);

    count++;

    clear(scrbuf);

if (key[KEY_F1]) plr1.special = MINE;
if (key[KEY_F2]) plr1.special = OIL;
if (key[KEY_F3]) plr1.special = SHIELD;

    if (colcheck(&plr1, &plr2)==1)
    {
      xv1 = abs(plr1.xvel + plr2.xvel);
      yv1 = abs(plr1.yvel + plr2.yvel);
      if (plr1.x < plr2.x)
      {
        plr1.xvel = -xv1/2;
        plr2.xvel = xv1/2;
      }
      else
      {
        plr1.xvel = xv1/2;
        plr2.xvel = -xv1/2;
      }
      if (plr1.y < plr2.y)
      {
        plr1.yvel = -yv1/2;
        plr2.yvel = yv1/2;
      }
      else
      {
        plr1.yvel = yv1/2;
        plr2.yvel = -yv1/2;
      }
      plr1.x += plr1.xvel*2;
      plr2.x += plr2.xvel*2;
      plr1.y += plr1.yvel*2;
      plr2.y += plr2.yvel*2;
    }

    if (plr1.lives<0 || plr2.lives < 0 || key[KEY_ESC]) done = 1;
  }
  list1.killlist();
  plist1.killlist();
  return;
}
/****************************************************************************/
void init_system()
{
  int i, i2;

  if (allegro_init()) { printf("Allegro Unavailable"); exit(1); }

  if (install_timer()) { printf("Timer Install Failed"); exit(1); }

  if (install_keyboard()) { printf("Keyboard Hook Failed"); exit(1); }

  if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, "./err.cfg")) { printf("Sound Init Failed"); exit(1); }

  if (set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0))
  { printf("Graphics Mode Unavailable"); exit(1); }

  text_mode(-1);

  LOCK_VARIABLE(fps);
  LOCK_FUNCTION(fps_timer);
  LOCK_VARIABLE(gologic);

  scrbuf = create_bitmap(576, 576);
  clear(scrbuf);

  panel[0] = load_bmp("./images/menus/bsidebar.bmp", pal);
  panel[1] = load_bmp("./images/menus/rsidebar.bmp", pal);
  panel[2] = load_bmp("./images/menus/gsidebar.bmp", pal);
  panel[3] = load_bmp("./images/menus/wsidebar.bmp", pal);

  healthbar = load_bmp("./images/menus/armorbar.bmp", pal);

  lowrezbg = create_bitmap(576, 576);

  // load car bitmaps
  car1 = load_bmp("./images/cars/car1.bmp", pal);
  car2 = load_bmp("./images/cars/car2.bmp", pal);
  car3 = load_bmp("./images/cars/car3.bmp", pal);
  car4 = load_bmp("./images/cars/car4.bmp", pal);

  // load weapon bitmaps
  machinegun = load_bmp("./images/weapons/machine.bmp", pal);
  bazooka = load_bmp("./images/weapons/bazooka.bmp", pal);
  laser = load_bmp("./images/weapons/laser.bmp", pal);
  cannon = load_bmp("./images/weapons/cannon.bmp", pal);
  nuke = load_bmp("./images/weapons/nuke.bmp", pal);
  rocketrack = load_bmp("./images/weapons/rokrak.bmp", pal);
  napalm = load_bmp("./images/weapons/napalm.bmp", pal);

  // load explosion bitmaps
  bazexp = load_bmp("./images/exps/bazexp.bmp", pal);
  lasexp = load_bmp("./images/exps/laserexp.bmp", pal);
  cannexp = load_bmp("./images/exps/cannexp.bmp", pal);
  nukeexp = load_bmp("./images/exps/nukeexp.bmp", pal);
  rrexp = load_bmp("./images/exps/rrexp.bmp", pal);
  napexp = load_bmp("./images/exps/napexp.bmp", pal);
  mineexp = load_bmp("./images/exps/mineexp.bmp", pal);
  carexp = load_bmp("./images/exps/carexp.bmp", pal);

  // load specials bitmaps
  mine = load_bmp("./images/specials/mine.bmp", pal);
  oil = load_bmp("./images/specials/oil.bmp", pal);
  shield = load_bmp("./images/specials/shield.bmp", pal);
  bot = load_bmp("./images/specials/bot.bmp", pal);

  // load gamefield bitmaps
  fixup = load_bmp("./images/game/fixup.bmp", pal);
  pitstop = load_bmp("./images/game/pitstop.bmp", pal);
  smallmoney = load_bmp("./images/game/money1.bmp", pal);
  bigmoney = load_bmp("./images/game/money2.bmp", pal);
  ammo = load_bmp("./images/game/ammo.bmp", pal);

  set_palette(pal);


  //create weapons sprites
  machinegun_spr.create(machinegun, 1);
  bazooka_spr.create(bazooka, 1);
  cannon_spr.create(cannon, 1);
  laser_spr.create(laser, 1);
  rocketrack_spr.create(rocketrack, 2);
  napalm_spr.create(napalm, 2);
  nuke_spr.create(nuke, 1);

  //create special and game sprites
  mine_spr.create(mine, 2);
  oil_spr.create(oil, 1);
  shield_spr.create(shield, 1);
  pitstop_spr.create(pitstop, 1);
  fixup_spr.create(fixup, 1);
  smallmoney_spr.create(smallmoney, 1);
  bigmoney_spr.create(bigmoney, 1);
  ammo_spr.create(ammo, 1);

  //create explosions sprites
  bazexp_spr.create(bazexp, 5);
  cannexp_spr.create(cannexp, 5);
  lasexp_spr.create(lasexp, 3);
  rrexp_spr.create(rrexp, 4);
  napexp_spr.create(napexp, 4);
  nukeexp_spr.create(nukeexp, 5);
  carexp_spr.create(carexp, 6);
  mineexp_spr.create(mineexp, 4);

  //create player sprites
  car1_spr.create(car1, 1);
  car2_spr.create(car2, 1);
  car3_spr.create(car3, 1);
  car4_spr.create(car4, 1);

  install_int(fps_timer, 1000);
  install_int(logic_timer, 10);
}

/****************************************************************************/

int colcheck(ent *obj1, ent *obj2)
{
  int i, i2, c, xdist, ydist, dist, coldist, xn1, xn2, yn1, yn2;

  xn1 = (int)(obj1->x+PLAYFIELDW)%PLAYFIELDW; xn2 = (int)(obj2->x+PLAYFIELDW)%PLAYFIELDW;

  yn1 = (int)(obj1->y+PLAYFIELDH)%PLAYFIELDH; yn2 = (int)(obj2->y+PLAYFIELDH)%PLAYFIELDH;

  xdist = abs(xn1-xn2);
  ydist = abs(yn1-yn2);

  dist = sqrt(xdist*xdist+ydist*ydist);
  coldist = obj1->sprit->radius[obj1->frame] + obj2->sprit->radius[obj2->frame];
  if (dist < coldist)
  {
    return 1;
  }
  else return 0;
}

/****************************************************************************/

int mapcolcheck(ent *obj)
{
  int ret=0;
  if (mag)
  {
  if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)]==1) ret += CENTER;
  if ((int)(obj->x/BLOCKSIZE) != (int)((obj->x+obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)+1][(int)(obj->y/BLOCKSIZE)]==1) ret += RIGHT;
  if ((int)(obj->x/BLOCKSIZE) != (int)((obj->x-obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)-1][(int)(obj->y/BLOCKSIZE)]==1) ret += LEFT;
  if ((int)(obj->y/BLOCKSIZE) != (int)((obj->y+obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)+1]==1) ret += BOTTOM;
  if ((int)(obj->y/BLOCKSIZE) != (int)((obj->y-obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)-1]==1) ret += TOP;
  }
  else
  {
  if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)]==1) ret += CENTER;
  if ((int)(obj->x/BLOCKSIZE) != (int)((obj->x+obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)+1][(int)(obj->y/BLOCKSIZE)]==1) ret += RIGHT;
  if ((int)(obj->x/BLOCKSIZE) != (int)((obj->x-obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)-1][(int)(obj->y/BLOCKSIZE)]==1) ret += LEFT;
  if ((int)(obj->y/BLOCKSIZE) != (int)((obj->y+obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)+1]==1) ret += BOTTOM;
  if ((int)(obj->y/BLOCKSIZE) != (int)((obj->y-obj->sprit->radius[obj->frame])/BLOCKSIZE))
    if (map[(int)(obj->x/BLOCKSIZE)][(int)(obj->y/BLOCKSIZE)-1]==1) ret += TOP;
  }
  return ret;
}

/****************************************************************************/

void update_panels(player plr1, player plr2)
{
  int i;

  text_mode(0);

  switch (plr1.special)
  {
  case NONE:
    rectfill(screen, 16, 168, 45, 193, 0);
    break;
  case MINE:
    blit(mine, screen, 0, 0, 19, 175, 24, 12);
    break;
  case OIL:
    blit(oil, screen, 0, 0, 18, 171, 29, 21);
    break;
  case SHIELD:
    int i;
    for (i=0; i<6; i++) draw_sprite(screen, shield, 28+scale_x(i*60)*8, 178+scale_y(i*60)*8);
    break;
  case BOT:
    blit(bot, screen, 0, 0, 19, 172, 27, 21);
    break;
  };

  switch (plr2.special)
  {
  case NONE:
    rectfill(screen, 704, 168, 733, 193, 0);
    break;
  case MINE:
    blit(mine, screen, 0, 0, 706, 172, 24, 12);
    break;
  case OIL:
    blit(oil, screen, 0, 0, 708, 172, 29, 21);
    break;
  case BOT:
    blit(bot, screen, 0, 0, 707, 172, 27, 21);
    break;
  };

  if (plr1.special) textprintf_centre(screen, font, 74, 176, 84, " %d ", (int)plr1.specammo[plr1.special]);
  else textprintf_centre(screen, font, 74, 176, 84, "    ");
  if (plr2.special) textprintf_centre(screen, font, 762, 176, 84, " %d ", (int)plr2.specammo[plr2.special]);
  else  textprintf_centre(screen, font, 762, 176, 84, "    ");

  textprintf(screen, font, 88, 18, 92, "%d", plr1.lives);
  textprintf(screen, font, 776, 18, 92, "%d", plr2.lives);

  rectfill(screen, 14, 12, 82, 33, 0);
  rectfill(screen, 702, 12, 770, 33, 0);
  textprintf_centre(screen, font, 46, 19, 92, "$%d", plr1.money);
  textprintf_centre(screen, font, 734, 19, 92, " $%d", plr2.money);

  rectfill(screen, 41, 114, 71, 122, 0);
  rectfill(screen, 729, 114, 760, 122, 0);
  textprintf(screen, font, 41, 114, 92, "%d%%", (int)plr1.life);
  textprintf(screen, font, 729, 114, 92, "%d%%", (int)plr2.life);

  rectfill(screen, 41, 65, 71, 65+41-(.42*abs(plr1.life)),0);
  blit(healthbar, screen, 0, 44-(.43*plr1.life), 41, 64+44-(.43*plr1.life), 28, (.43*plr1.life));

  rectfill(screen, 729, 65, 759, 65+41-(.42*abs(plr2.life)),0);
  blit(healthbar, screen, 0, 44-(.43*plr2.life), 729, 64+44-(.43*plr2.life), 28, (.43*plr2.life));

  for (i=1; i<7; i++)
  {
    textprintf_centre(screen, font, 74, 256+51*i, 84, " %d ", (int)plr1.ammo[i]);
  }
  if (plr1.weapon>MACHINE_GUN)
  {
    textprintf_centre(screen, font, 74, 256+51*plr1.weapon, 92, " %d ", plr1.ammo[plr1.weapon]);
    line(screen, 58, 262, 90, 262, 84);
  }
  else line(screen, 58, 262, 90, 262, 92);

  for (i=1; i<7; i++)
  {
    textprintf_centre(screen, font, 762, 256+51*i, 84, " %d ", (int)plr2.ammo[i]);
  }
  if (plr2.weapon>MACHINE_GUN)
  {
    textprintf_centre(screen, font, 762, 256+51*plr2.weapon, 92, " %d ", plr2.ammo[plr2.weapon]);
    line(screen, 746, 262, 778, 262, 84);
  }
  else line(screen, 746, 262, 778, 262, 92);

  text_mode(-1);
}

int load_level(char *fname)
{
  int i;

  mapfile = pack_fopen(fname, "r");
  for (i=0; i<36*36; i++) map[i%36][i/36] = pack_getc(mapfile);
  pack_fclose(mapfile);
  return 0;
}
RGB getcol(int col)
{
  RGB c;
  get_color(col, &c);
  return c;
}
void palrot()
{
  RGB *bak;
  get_color(217, bak);
  set_color(217, &getcol(216));
  set_color(216, &getcol(215));
  set_color(215, &getcol(214));
  set_color(214, &getcol(213));
  set_color(213, &getcol(212));
  set_color(212, &getcol(211));
  set_color(211, &getcol(210));
  set_color(210, &getcol(209));
  set_color(209, &getcol(208));
  set_color(208, bak);
}
/****************************************************************************/
  BITMAP *screenbuf;
  BITMAP *BattleTruck;
  BITMAP *PlayGame;
  BITMAP *Option;
  BITMAP *Controls;
  BITMAP *Credits;
  BITMAP *Quit;

void do_credits()
{
  BITMAP *creds = load_bmp("./images/menus/creds.bmp", pal);

  clear(screen);
  draw_sprite(screen, creds, 200, 100);

  textprintf_centre(screen, font, 400, 500, 30, "Press ESC to return to the menu");

  while (!key[KEY_ESC]);
  destroy_bitmap(creds);
}
/****************************************************************************/
void pick_cars()
{
  int car1=-1, car2=-1;
  BITMAP *scrnbuf = create_bitmap(800, 600);
  BITMAP *cars = load_bmp("./images/menus/cars.bmp", pal);
  int select = 0, selectdelay = 0;

  rest(200);

  while (car1==-1)
  {
    if (key[KEY_ESC]) return;
    clear(scrnbuf);
    draw_sprite(scrnbuf, PlayGame, 320, 20);
    textprintf_centre(scrnbuf, font, 400, 180, 90, "Player 1: Select your vehicle");

    textprintf_centre(scrnbuf, font, 400, 450, 30, "Press Left and Right to select a car");
    textprintf_centre(scrnbuf, font, 400, 460, 30, "Press Enter to choose that car");
    textprintf_centre(scrnbuf, font, 400, 500, 30, "Press ESC to return to the menu");

    draw_sprite(scrnbuf, cars, 150, 250);

    rect(scrnbuf, 140+100*select, 250, 190+100*select, 300, 30);

    if (key[KEY_RIGHT] && (!selectdelay)) { select++; selectdelay = 10; }
    if (key[KEY_LEFT] && (!selectdelay)) { select--; selectdelay = 10; }

    if (key[KEY_ENTER] && (!selectdelay))
    {
      selectdelay = 50;
      car1 = select;
    }

    select = (select+4)%4;
    if (selectdelay) selectdelay--;

    blit(scrnbuf, screen, 0, 0, 0, 0, 800, 600);
  }

  while (car2==-1)
  {
    if (key[KEY_ESC]) return;
    clear(scrnbuf);
    draw_sprite(scrnbuf, PlayGame, 320, 20);
    textprintf_centre(scrnbuf, font, 400, 180, 90, "Player 2: Select your vehicle");

    textprintf_centre(scrnbuf, font, 400, 450, 30, "Press Left and Right to select a car");
    textprintf_centre(scrnbuf, font, 400, 460, 30, "Press Enter to choose that car");
    textprintf_centre(scrnbuf, font, 400, 500, 30, "Press ESC to return to the menu");

    draw_sprite(scrnbuf, cars, 150, 250);

    rect(scrnbuf, 140+100*select, 250, 190+100*select, 300, 30);

    if (key[KEY_RIGHT] && (!selectdelay)) { select++; selectdelay = 10; }
    if (key[KEY_LEFT] && (!selectdelay)) { select--; selectdelay = 10; }

    if (key[KEY_ENTER] && (!selectdelay))
    {
      selectdelay = 50;
      car2 = select;
    }

    select = (select+4)%4;
    if (selectdelay) selectdelay--;

    blit(scrnbuf, screen, 0, 0, 0, 0, 800, 600);
  }
  destroy_bitmap(scrnbuf);
  destroy_bitmap(cars);
  playgame(car1, car2, 0);
  rest(200);
}
/****************************************************************************/
void do_options()
{
  BITMAP *screnbuf = create_bitmap(800, 600);
  int select = 0, selectdelay = 0;

  while (!key[KEY_ESC])
  {
    clear(screnbuf);
    draw_sprite(screnbuf, Option, 250, 20);
    textprintf_centre(screnbuf, font, 400, 450, 30, "Press Up and Down to navigate the options");
    textprintf_centre(screnbuf, font, 400, 460, 30, "Press Enter to change a setting");
    textprintf_centre(screnbuf, font, 400, 500, 30, "Press ESC to return to the menu");

    if (select != 0) textprintf_centre(screnbuf, font, 200, 200, 70, "Zooming Camera: ");
    else textprintf_centre(screnbuf, font, 200, 200, 80, "Zooming Camera: ");
    if (nomag) textprintf_centre(screnbuf, font, 600, 200, 70, "NO");
    else textprintf_centre(screnbuf, font, 600, 200, 70, "YES");

    if (select != 1) textprintf_centre(screnbuf, font, 200, 220, 70, "Money Frequency: ");
    else textprintf_centre(screnbuf, font, 200, 220, 80, "Money Frequency: ");
    if (moneyrate==1000) textprintf_centre(screnbuf, font, 600, 220, 70, "HIGH");
    if (moneyrate==2000) textprintf_centre(screnbuf, font, 600, 220, 70, "MEDIUM");
    if (moneyrate==5000) textprintf_centre(screnbuf, font, 600, 220, 70, "LOW");
    if (moneyrate==-1) textprintf_centre(screnbuf, font, 600, 220, 70, "OFF");

    if (select != 2) textprintf_centre(screnbuf, font, 200, 240, 70, "Wrench Frequency: ");
    else textprintf_centre(screnbuf, font, 200, 240, 80, "Wrench Frequency: ");
    if (fixuprate==1000) textprintf_centre(screnbuf, font, 600, 240, 70, "HIGH");
    if (fixuprate==2000) textprintf_centre(screnbuf, font, 600, 240, 70, "MEDIUM");
    if (fixuprate==5000) textprintf_centre(screnbuf, font, 600, 240, 70, "LOW");
    if (fixuprate==-1) textprintf_centre(screnbuf, font, 600, 240, 70, "OFF");

    if (select != 3) textprintf_centre(screnbuf, font, 200, 260, 70, "Ammo Frequency: ");
    else textprintf_centre(screnbuf, font, 200, 260, 80, "Ammo Frequency: ");
    if (ammorate==1000) textprintf_centre(screnbuf, font, 600, 260, 70, "HIGH");
    if (ammorate==2000) textprintf_centre(screnbuf, font, 600, 260, 70, "MEDIUM");
    if (ammorate==5000) textprintf_centre(screnbuf, font, 600, 260, 70, "LOW");
    if (ammorate==-1) textprintf_centre(screnbuf, font, 600, 260, 70, "OFF");

    if (key[KEY_DOWN] && (!selectdelay)) { select++; selectdelay = 10; }
    if (key[KEY_UP] && (!selectdelay)) { select--; selectdelay = 10; }

    if (key[KEY_ENTER] && (!selectdelay))
    {
      selectdelay = 50;
      switch (select)
      {
      case 0:
        nomag ^= 1;
      break;
      case 1:
        if (moneyrate==1000) moneyrate = 2000;
        else if (moneyrate==2000) moneyrate = 5000;
        else if (moneyrate==5000) moneyrate = -1;
        else if (moneyrate==-1) moneyrate = 1000;
      break;
      case 2:
        if (fixuprate==1000) fixuprate = 2000;
        else if (fixuprate==2000) fixuprate = 5000;
        else if (fixuprate==5000) fixuprate = -1;
        else if (fixuprate==-1) fixuprate = 1000;
      break;
      case 3:
        if (ammorate==1000) ammorate = 2000;
        else if (ammorate==2000) ammorate = 5000;
        else if (ammorate==5000) ammorate = -1;
        else if (ammorate==-1) ammorate = 1000;
      break;
      }
    }

    select = (select+4)%4;
    if (selectdelay) selectdelay--;

    blit(screnbuf, screen, 0, 0, 0, 0, 800, 600);
  }
  destroy_bitmap(screnbuf);
}
/****************************************************************************/
void do_controls()
{
  clear(screen);
  draw_sprite(screen, Controls, 250, 20);
  textprintf_centre(screen, font, 300, 150, 90, "Player 1:");
  textprintf_centre(screen, font, 300, 160, 90, "Turn Left: A");
  textprintf_centre(screen, font, 300, 170, 90, "Turn Right: A");
  textprintf_centre(screen, font, 300, 180, 90, "Move Forward: W");
  textprintf_centre(screen, font, 300, 200, 90, "Move Back: S");
  textprintf_centre(screen, font, 300, 210, 90, "Fire Weapon: F");
  textprintf_centre(screen, font, 300, 220, 90, "Use Special: G");
  textprintf_centre(screen, font, 300, 230, 90, "Cycle Weapons: CAPSLOCK & LSHIFT");

  textprintf_centre(screen, font, 600, 150, 90, "Player 2:");
  textprintf_centre(screen, font, 600, 160, 90, "Turn Left: DELETE");
  textprintf_centre(screen, font, 600, 170, 90, "Turn Right: PAGE DOWN");
  textprintf_centre(screen, font, 600, 180, 90, "Move Forward: HOME");
  textprintf_centre(screen, font, 600, 200, 90, "Move Back: END");
  textprintf_centre(screen, font, 600, 210, 90, "Fire Weapon: NUMLOCK");
  textprintf_centre(screen, font, 600, 220, 90, "Use Special: FORWARDSLASH");
  textprintf_centre(screen, font, 600, 230, 90, "Cycle Weapons: BACKSPACE & BACKSLASH");

  textprintf_centre(screen, font, 400, 300, 30, "Sorry about the screwy controls, but");
  textprintf_centre(screen, font, 400, 310, 30, "due to the limitations of PC keyboards");
  textprintf_centre(screen, font, 400, 320, 30, "this is the only working combination I could find");

  textprintf_centre(screen, font, 400, 500, 30, "Press ESC to return to the menu");

  while (!key[KEY_ESC]);
}
/****************************************************************************/
int do_menu()
{
  int done = 0;

  screenbuf = create_bitmap(800, 600);
  clear(screenbuf);

  BattleTruck = load_bmp("./images/menus/battle.bmp", pal);
  PlayGame = load_bmp("./images/menus/play.bmp", pal);
  Option = load_bmp("./images/menus/options.bmp", pal);
  Controls = load_bmp("./images/menus/controls.bmp", pal);
  Credits = load_bmp("./images/menus/credits.bmp", pal);
  Quit = load_bmp("./images/menus/quit.bmp", pal);

  set_palette(pal);

  int select = 0, selectdelay = 0;

  while (!done)
  {
    clear(screenbuf);

    if (key[KEY_ESC]) done = 1;
    if (key[KEY_DOWN] && (!selectdelay)) { select++; selectdelay = 10; }
    if (key[KEY_UP] && (!selectdelay)) { select--; selectdelay = 10; }

    if (key[KEY_ENTER])
    {
      switch (select)
      {
      case 0:
        plist1.killlist();
        pick_cars();
        rest(200);
      break;
      case 1:
        plist1.killlist();
        do_options();
        rest(300);
      break;
      case 2:
        plist1.killlist();
        do_controls();
        rest(300);
      break;
      case 3:
        plist1.killlist();
        do_credits();
        rest(300);
      break;
      case 4:
        done = 1;
      break;
      }
    }

    select = (select+5)%5;
    if (selectdelay) selectdelay--;

    draw_sprite(screenbuf, BattleTruck, 150, 20);
    draw_sprite(screenbuf, PlayGame, 320, 200);
    draw_sprite(screenbuf, Option, 265, 280);
    draw_sprite(screenbuf, Controls, 250, 360);
    draw_sprite(screenbuf, Credits, 265, 440);
    draw_sprite(screenbuf, Quit, 320, 515);

    rotate_sprite(screenbuf, bazooka, 150, 210+80*select, itofix(64));
   plist1.addParticle(145, 210+80*select+15, scale_x(-90+rand()%40-20)*2,
                                  scale_y(-90+rand()%40-20)*2, 0, par_flamego);

    rotate_sprite(screenbuf, bazooka, 650, 210+80*select, itofix(192));
   plist1.addParticle(670, 210+80*select+15, scale_x(90+rand()%40-20)*2,
                                  scale_y(90+rand()%40-20)*2, 0, par_flamego);


    plist1.movelist();
    plist1.drawlist(screenbuf, 0, 0);
    blit(screenbuf, screen, 0, 0, 0, 0, 800, 600);
  }
}
